"use strict";
var osenv = require("osenv");
var path = require("path");

var PostInstallCommand = (function () {
    function PostInstallCommand($fs, $childProcess, $logger, $staticConfig) {
        this.$fs = $fs;
        this.$childProcess = $childProcess;
        this.$logger = $logger;
        this.$staticConfig = $staticConfig;
        this.disableAnalytics = true;
    }
    PostInstallCommand.prototype.execute = function (args) {
        var _this = this;
        return (function () {
            if (process.platform !== "win32") {
                _this.$fs.chmod(_this.$staticConfig.adbFilePath, "0777").wait();
            }

            _this.enableAutoCompletion().wait();
        }).future()();
    };

    PostInstallCommand.prototype.enableAutoCompletion = function () {
        var _this = this;
        return (function () {
            var scriptsOk = true;

            try  {
                _this.updateShellScript(".profile").wait();
                _this.updateShellScript(".bashrc").wait();

                _this.updateBashProfile().wait();

                _this.updateShellScript(".zshrc").wait();
            } catch (err) {
                _this.$logger.out("Failed to update all shell start-up scripts. Auto-completion may not work. " + err);
                scriptsOk = false;
            }

            if (scriptsOk) {
                _this.$logger.out("Restart your shell to enable command auto-completion.");
            }
        }).future()();
    };

    PostInstallCommand.prototype.updateShellScript = function (fileName) {
        var _this = this;
        return (function () {
            var filePath = _this.getHomePath(fileName);

            var doUpdate = true;
            if (_this.$fs.exists(filePath).wait()) {
                var contents = _this.$fs.readText(filePath).wait();
                if (contents.match(/nativescript\s+completion\s+--\s+/) || contents.match(/tns\s+completion\s+--\s+/)) {
                    doUpdate = false;
                }
            }

            if (doUpdate) {
                _this.updateShellScriptCore(filePath).wait();
            }
        }).future()();
    };

    PostInstallCommand.prototype.updateShellScriptCore = function (filePath) {
        var _this = this;
        return (function () {
            _this.$childProcess.exec("nativescript completion >> " + filePath).wait();
        }).future()();
    };

    PostInstallCommand.prototype.getHomePath = function (fileName) {
        return path.join(osenv.home(), fileName);
    };

    PostInstallCommand.prototype.updateBashProfile = function () {
        var _this = this;
        return (function () {
            var bashProfileFileName = _this.getHomePath(".bash_profile");
            if (_this.$fs.exists(bashProfileFileName).wait()) {
                var contens = _this.$fs.readText(bashProfileFileName).wait();
                if (contens.indexOf(PostInstallCommand.CALL_PROFILE_SCRIPT) < 0) {
                    _this.updateShellScript(".bash_profile");
                }
            } else {
                _this.$fs.writeFile(bashProfileFileName, PostInstallCommand.CALL_PROFILE_SCRIPT).wait();
                _this.updateShellScriptCore(bashProfileFileName).wait();
            }
        }).future()();
    };
    PostInstallCommand.CALL_PROFILE_SCRIPT = "if [ -f ~/.profile ]; then\n" + "		. ~/.profile\n" + "fi\n";
    return PostInstallCommand;
})();
exports.PostInstallCommand = PostInstallCommand;
$injector.registerCommand("dev-post-install", PostInstallCommand);
