"use strict";
var MobileHelper = require("./../mobile-helper");
var util = require("util");

var path = require("path");
var temp = require("temp");
var byline = require("byline");
var helpers = require("../../helpers");
var os = require("os");

var AndroidDevice = (function () {
    function AndroidDevice(identifier, adb, $logger, $fs, $childProcess, $errors, $propertiesParser, $staticConfig) {
        this.identifier = identifier;
        this.adb = adb;
        this.$logger = $logger;
        this.$fs = $fs;
        this.$childProcess = $childProcess;
        this.$errors = $errors;
        this.$propertiesParser = $propertiesParser;
        this.$staticConfig = $staticConfig;
        this._tmpRoot = Object.create(null);
        var details = this.getDeviceDetails().wait();
        this.model = details.model;
        this.name = details.name;
        this.version = details.release;
        this.vendor = details.brand;
    }
    AndroidDevice.prototype.getDeviceDetails = function () {
        var _this = this;
        return (function () {
            var requestDeviceDetailsCommand = _this.composeCommand("shell cat /system/build.prop");
            var details = _this.$childProcess.exec(requestDeviceDetailsCommand).wait();

            var parsedDetails = {};
            details.split(/\r?\n|\r/).forEach(function (value) {
                var match = /(?:ro\.build\.version|ro\.product)\.(.+)=(.+)/.exec(value);
                if (match) {
                    parsedDetails[match[1]] = match[2];
                }
            });

            return parsedDetails;
        }).future()();
    };

    AndroidDevice.prototype.getPlatform = function () {
        return MobileHelper.DevicePlatforms[1 /* Android */];
    };

    AndroidDevice.prototype.getIdentifier = function () {
        return this.identifier;
    };

    AndroidDevice.prototype.getDisplayName = function () {
        return this.name;
    };

    AndroidDevice.prototype.getModel = function () {
        return this.model;
    };

    AndroidDevice.prototype.getVersion = function () {
        return this.version;
    };

    AndroidDevice.prototype.getVendor = function () {
        return this.vendor;
    };

    AndroidDevice.prototype.getInstalledApplications = function () {
        var _this = this;
        return (function () {
            if (!_this._installedApplications) {
                var listPackagesCommand = _this.composeCommand("shell pm list packages");
                var result = _this.$childProcess.exec(listPackagesCommand).wait();
                _this._installedApplications = _.map(result.split(os.EOL), function (packageString) {
                    var match = packageString.match(/package:(.+)/);
                    return match ? match[1] : null;
                }).filter(function (parsedPackage) {
                    return parsedPackage != null;
                });
            }

            return _this._installedApplications;
        }).future()();
    };

    AndroidDevice.prototype.composeCommand = function () {
        var args = [];
        for (var _i = 0; _i < (arguments.length - 0); _i++) {
            args[_i] = arguments[_i + 0];
        }
        var command = util.format.apply(null, args);
        var result = util.format("\"%s\" -s %s", this.adb, this.identifier);
        if (command && !command.isEmpty()) {
            result += util.format(" %s", command);
        }

        return result;
    };

    AndroidDevice.prototype.startPackageOnDevice = function (packageName) {
        var _this = this;
        return (function () {
            var startPackageCommand = _this.composeCommand("shell am start -a android.intent.action.MAIN -n %s/%s", packageName, _this.$staticConfig.START_PACKAGE_ACTIVITY_NAME);
            var result = _this.$childProcess.exec(startPackageCommand).wait();
            return result[0];
        }).future()();
    };

    AndroidDevice.prototype.deploy = function (packageFile, packageName) {
        var _this = this;
        return (function () {
            var uninstallCommand = _this.composeCommand("shell pm uninstall \"%s\"", packageName);
            _this.$childProcess.exec(uninstallCommand).wait();

            var installCommand = _this.composeCommand("install -r \"%s\"", packageFile);
            _this.$childProcess.exec(installCommand).wait();

            _this.startPackageOnDevice(packageName).wait();
            _this.$logger.info("Successfully deployed on device with identifier '%s'", _this.getIdentifier());
        }).future()();
    };

    AndroidDevice.prototype.prepareTmpDir = function (appIdentifier) {
        var _this = this;
        return (function () {
            if (!_this._tmpRoot[appIdentifier.appIdentifier]) {
                var tmpRoot = _this.buildDevicePath(AndroidDevice.DEVICE_TMP_DIR, AndroidDevice.SYNC_ROOT, appIdentifier.appIdentifier);
                var filesInTmp = _this.buildDevicePath(tmpRoot, "*");

                var command = _this.composeCommand('shell mkdir -p "%s"', tmpRoot);
                _this.$childProcess.exec(command).wait();
                command = _this.composeCommand('shell rm -rf "%s"', filesInTmp);
                _this.$childProcess.exec(command).wait();
                _this._tmpRoot[appIdentifier.appIdentifier] = tmpRoot;
            }
            return _this._tmpRoot[appIdentifier.appIdentifier];
        }).future()();
    };

    AndroidDevice.prototype.pushFilesOnDevice = function (localToDevicePaths, appIdentifier) {
        var _this = this;
        return (function () {
            var tmpRoot = _this.prepareTmpDir(appIdentifier).wait();

            localToDevicePaths.forEach(function (localToDevicePathData) {
                var tmpPath = _this.buildDevicePath(tmpRoot, helpers.fromWindowsRelativePathToUnix(localToDevicePathData.getRelativeToProjectBasePath()));
                _this.pushFileOnDevice(localToDevicePathData.getLocalPath(), tmpPath).wait();
            });

            var adbCommand = util.format('IFS=$\'\\n\'; for i in $(ls -a %s); do rm -rf %s/$i && mv %s/$i %s; done; unset IFS', tmpRoot, appIdentifier.deviceProjectPath, tmpRoot, appIdentifier.deviceProjectPath);
            _this.$childProcess.execFile(_this.adb, [
                "-s", _this.identifier,
                "shell",
                adbCommand
            ]).wait();
        }).future()();
    };

    AndroidDevice.prototype.pushFileOnDevice = function (localPath, devicePath) {
        var _this = this;
        return (function () {
            var rmCommand = _this.composeCommand('shell rm -r "%s"', devicePath);
            _this.$childProcess.exec(rmCommand).wait();

            if (_this.$fs.exists(localPath).wait()) {
                var isDirectory = _this.$fs.getFsStats(localPath).wait().isDirectory();

                var mkdirCommand = _this.composeCommand('shell mkdir -p "%s"', isDirectory ? devicePath : path.dirname(devicePath));
                _this.$childProcess.exec(mkdirCommand).wait();

                var pushFileCommand = _this.composeCommand('push "%s" "%s"', isDirectory ? path.join(localPath, ".") : localPath, devicePath);
                _this.$childProcess.exec(pushFileCommand).wait();
            }
        }).future()();
    };

    AndroidDevice.prototype.sendBroadcastToDevice = function (action, extras) {
        var _this = this;
        if (typeof extras === "undefined") { extras = {}; }
        return (function () {
            var broadcastCommand = _this.composeCommand("shell am broadcast -a \"%s\"", action);

            Object.keys(extras).forEach(function (key) {
                broadcastCommand += util.format(" -e \"%s\" \"%s\"", key, extras[key]);
            });

            var result = _this.$childProcess.exec(broadcastCommand).wait();
            var match = result.match(/Broadcast completed: result=(\d+)/);
            if (match) {
                return +match[1];
            } else {
                _this.$errors.fail("Unable to broadcast to android device:\n%s", result);
            }
        }).future()();
    };

    AndroidDevice.prototype.getLiveSyncUrl = function (projectType) {
        var projectTypes = $injector.resolve("$projectTypes");
        switch (projectType) {
            case projectTypes.Cordova:
                return "icenium://";
            case projectTypes.NativeScript:
                return "nativescript://";
            default:
                throw new Error("Unsupported project type");
        }
    };

    AndroidDevice.prototype.sync = function (localToDevicePaths, appIdentifier, projectType, options) {
        var _this = this;
        if (typeof options === "undefined") { options = {}; }
        return (function () {
            if (appIdentifier.isLiveSyncSupported(_this).wait()) {
                var ionVer = _this.getIonVersion(appIdentifier).wait();
                if (_this.isNewProtocol(ionVer, AndroidDevice.ION_VER_NEW_SYNC_PROTOCOL)) {
                    _this.syncNewProtocol(localToDevicePaths, appIdentifier, projectType, options).wait();
                } else {
                    _this.syncOldProtocol(localToDevicePaths, appIdentifier, projectType, options).wait();
                }
                _this.$logger.info("Successfully synced device with identifier '%s'", _this.getIdentifier());
            } else {
                _this.$errors.fail({ formatStr: appIdentifier.getliveSyncNotSupportedError(_this), suppressCommandHelp: true });
            }
        }).future()();
    };

    AndroidDevice.prototype.syncOldProtocol = function (localToDevicePaths, appIdentifier, projectType, options) {
        var _this = this;
        if (typeof options === "undefined") { options = {}; }
        return (function () {
            _this.pushFilesOnDevice(localToDevicePaths, appIdentifier).wait();
            if (!options.skipRefresh) {
                var changeLiveSyncUrlExtras = {
                    "liveSyncUrl": _this.getLiveSyncUrl(projectType),
                    "app-id": appIdentifier.appIdentifier
                };
                _this.sendBroadcastToDevice(AndroidDevice.CHANGE_LIVESYNC_URL_INTENT_NAME, changeLiveSyncUrlExtras).wait();
                _this.sendBroadcastToDevice(AndroidDevice.REFRESH_WEB_VIEW_INTENT_NAME, { "app-id": appIdentifier.appIdentifier }).wait();
            }
        }).future()();
    };

    AndroidDevice.prototype.getTempDir = function () {
        temp.track();
        return temp.mkdirSync("ab-");
    };

    AndroidDevice.prototype.syncNewProtocol = function (localToDevicePaths, appIdentifier, projectType, options) {
        var _this = this;
        if (typeof options === "undefined") { options = {}; }
        return (function () {
            _this.pushFilesOnDevice(localToDevicePaths, appIdentifier).wait();

            var hostTmpDir = _this.getTempDir();
            var commandsFileHostPath = path.join(hostTmpDir, AndroidDevice.COMMANDS_FILE);
            var commandsFile = _this.$fs.createWriteStream(commandsFileHostPath);
            var fileWritten = _this.$fs.futureFromEvent(commandsFile, 'finish');
            commandsFile.write("DeployProject " + _this.getLiveSyncUrl(projectType) + '\r');
            commandsFile.write("ReloadStartView" + '\r');
            commandsFile.end();
            fileWritten.wait();

            var commandsFileDevicePath = _this.buildDevicePath(_this.prepareTmpDir(appIdentifier).wait(), AndroidDevice.COMMANDS_FILE);
            _this.pushFileOnDevice(commandsFileHostPath, commandsFileDevicePath).wait();

            _this.sendBroadcastToDevice(AndroidDevice.LIVESYNC_BROADCAST_NAME, { "app-id": appIdentifier.appIdentifier }).wait();
        }).future()();
    };

    AndroidDevice.prototype.getIonVersion = function (appIdentifier) {
        var _this = this;
        return (function () {
            var output = _this.$childProcess.execFile(_this.adb, [
                "-s", _this.getIdentifier(),
                "shell", "dumpsys", "package", appIdentifier.appIdentifier
            ]).wait();
            return _this.$propertiesParser.parse(output)["versionName"];
        }).future()();
    };

    AndroidDevice.prototype.isNewProtocol = function (currentIonVersion, newProtocolversion) {
        return helpers.versionCompare(currentIonVersion, newProtocolversion) >= 0;
    };

    AndroidDevice.prototype.buildDevicePath = function () {
        var args = [];
        for (var _i = 0; _i < (arguments.length - 0); _i++) {
            args[_i] = arguments[_i + 0];
        }
        return args.join("/");
    };

    AndroidDevice.prototype.openDeviceLogStream = function () {
        var _this = this;
        var adbLogcat = this.$childProcess.spawn(this.adb, ["-s", this.getIdentifier(), "logcat"]);
        var lineStream = byline(adbLogcat.stdout);

        adbLogcat.stderr.on("data", function (data) {
            _this.$logger.trace("ADB logcat stderr: " + data.toString());
        });

        adbLogcat.on("close", function (code) {
            if (code !== 0) {
                _this.$logger.trace("ADB process exited with code " + code.toString());
            }
        });

        lineStream.on('data', function (line) {
            var lineText = line.toString();
            var log = _this.getConsoleLogFromLine(lineText);
            if (log) {
                if (log.tag) {
                    _this.$logger.out("%s: %s", log.tag, log.message);
                } else {
                    _this.$logger.out(log.message);
                }
            }
        });
    };

    AndroidDevice.prototype.getConsoleLogFromLine = function (lineText) {
        var acceptedTags = ["chromium", "Web Console"];

        var match = lineText.match(/.\/(.+?)\(\s*(\d+?)\): (.*)/);
        if (match) {
            if (acceptedTags.indexOf(match[1]) !== -1) {
                return { tag: match[1], message: match[3] };
            }
        } else if (_.any(acceptedTags, function (tag) {
            return lineText.indexOf(tag) !== -1;
        })) {
            return { message: match[3] };
        }

        return null;
    };
    AndroidDevice.REFRESH_WEB_VIEW_INTENT_NAME = "com.telerik.RefreshWebView";
    AndroidDevice.CHANGE_LIVESYNC_URL_INTENT_NAME = "com.telerik.ChangeLiveSyncUrl";
    AndroidDevice.LIVESYNC_BROADCAST_NAME = "com.telerik.LiveSync";

    AndroidDevice.DEVICE_TMP_DIR = "/data/local/tmp";
    AndroidDevice.ION_VER_NEW_SYNC_PROTOCOL = "2.5";
    AndroidDevice.SYNC_ROOT = "12590FAA-5EDD-4B12-856D-F52A0A1599F2";
    AndroidDevice.COMMANDS_FILE = "telerik.livesync.commands";
    return AndroidDevice;
})();
exports.AndroidDevice = AndroidDevice;
