var Future = require("fibers/future");
var npm = require("npm");
var path = require("path");
var semver = require("semver");

var helpers = require("./common/helpers");
var constants = require("./constants");

var NodePackageManager = (function () {
    function NodePackageManager($logger, $errors, $httpClient, $staticConfig) {
        this.$logger = $logger;
        this.$errors = $errors;
        this.$httpClient = $httpClient;
        this.$staticConfig = $staticConfig;
    }
    Object.defineProperty(NodePackageManager.prototype, "cache", {
        get: function () {
            return npm.cache;
        },
        enumerable: true,
        configurable: true
    });

    NodePackageManager.prototype.load = function (config) {
        var future = new Future();
        npm.load(config, function (err) {
            if (err) {
                future.throw(err);
            } else {
                future.return();
            }
        });
        return future;
    };

    NodePackageManager.prototype.install = function (packageName, pathToSave, version) {
        var _this = this;
        return (function () {
            try  {
                _this.load().wait();
                pathToSave = pathToSave || npm.cache;
                var packageToInstall = packageName;

                if (version) {
                    _this.validateVersion(packageName, version).wait();
                    packageToInstall = packageName + "@" + version;
                }

                _this.installCore(packageToInstall, pathToSave).wait();
            } catch (error) {
                _this.$logger.debug(error);
                _this.$errors.fail(NodePackageManager.NPM_LOAD_FAILED);
            }

            return path.join(pathToSave, "node_modules", packageName);
        }).future()();
    };

    NodePackageManager.prototype.installCore = function (packageName, pathToSave) {
        var currentVersion = this.$staticConfig.version;
        if (!semver.valid(currentVersion)) {
            this.$errors.fail("Invalid version.");
        }

        var incrementedVersion = semver.inc(currentVersion, constants.ReleaseType.MINOR);
        if (packageName.indexOf("@") < 0) {
            packageName = packageName + "@<" + incrementedVersion;
        }
        this.$logger.trace("Installing", packageName);

        var future = new Future();
        npm.commands["install"](pathToSave, packageName, function (err, data) {
            if (err) {
                future.throw(err);
            } else {
                future.return(data);
            }
        });
        return future;
    };

    NodePackageManager.prototype.getAvailableVersions = function (packageName) {
        var _this = this;
        return (function () {
            var url = NodePackageManager.NPM_REGISTRY_URL + packageName;
            var response = _this.$httpClient.httpRequest(url).wait().body;
            var json = JSON.parse(response);
            return _.keys(json.versions);
        }).future()();
    };

    NodePackageManager.prototype.validateVersion = function (packageName, version) {
        var _this = this;
        return (function () {
            var versions = _this.getAvailableVersions(packageName).wait();
            if (!_.contains(versions, version)) {
                _this.$errors.fail("Invalid version. Valid versions are: %s", helpers.formatListOfNames(versions, "and"));
            }
        }).future()();
    };
    NodePackageManager.NPM_LOAD_FAILED = "Failed to retrieve data from npm. Please try again a little bit later.";
    NodePackageManager.NPM_REGISTRY_URL = "http://registry.npmjs.org/";
    return NodePackageManager;
})();
exports.NodePackageManager = NodePackageManager;
$injector.register("npm", NodePackageManager);
